"""
Luna3 exceptions
"""
from typing import Any, Optional, Union

import ujson as json

from .luna_request import LunaRequest


class LunaApiException(Exception):
    """
    Exception for failed requests.

    Attributes:
        error (Any): optional error
        statusCode (int): http status code
        body (Union[bytes, str, dict]): response body
        request (LunaRequest): request
    """

    def __init__(self, message: str, response: "LunaResponse", lunaAPIError: Optional[Any] = None):
        super().__init__(message)
        self.error: Any = lunaAPIError
        self.statusCode: int = response.statusCode
        self.body: Union[bytes, str, dict] = response.body
        self.request: LunaRequest = response.request

    @property
    def json(self) -> Union[int, list, dict]:
        """
        Get error from body as json
        Returns:
            self. body if body is dict otherwise load from body
        """
        if isinstance(self.body, dict):
            return self.body
        return json.loads(self.body)

    def __repr__(self) -> str:
        """
        Get representation

        Returns:
            f"status code: {self.statusCode}, error: {self.json}"
        """
        return f"status code: {self.statusCode}, error: {self.json}"
